/* *********************************************************************** *
 * project: org.matsim.*
 * SignalPlanDataImpl
 *                                                                         *
 * *********************************************************************** *
 *                                                                         *
 * copyright       : (C) 2010 by the members listed in the COPYING,        *
 *                   LICENSE and WARRANTY file.                            *
 * email           : info at matsim dot org                                *
 *                                                                         *
 * *********************************************************************** *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *   See also COPYING, LICENSE and WARRANTY file                           *
 *                                                                         *
 * *********************************************************************** */
package org.matsim.signalsystems.data.signalcontrol.v20;

import java.util.SortedMap;
import java.util.TreeMap;

import org.matsim.api.core.v01.Id;


/**
 * @author dgrether
 *
 */
public class SignalPlanDataImpl implements SignalPlanData {

	private Id id;
	private Integer cycletime;
	private Double endtime;
	private Integer offset;
	private SortedMap<Id, SignalGroupSettingsData> signalGroupSettingsBySignalGroupId;
	private Double starttime;

	public SignalPlanDataImpl(Id id) {
		this.id = id;
	}

	@Override
	public void addSignalGroupSettings(SignalGroupSettingsData signalGroupSettings) {
		if (this.signalGroupSettingsBySignalGroupId == null) {
			this.signalGroupSettingsBySignalGroupId = new TreeMap<Id, SignalGroupSettingsData>();
		}
		this.signalGroupSettingsBySignalGroupId.put(signalGroupSettings.getSignalGroupId(), signalGroupSettings);
	}

	@Override
	public Integer getCycleTime() {
		return this.cycletime;
	}

	@Override
	public Double getEndTime() {
		return this.endtime;
	}

	@Override
	public Id getId() {
		return this.id;
	}

	@Override
	public Integer getOffset() {
		return this.offset;
	}

	@Override
	public SortedMap<Id, SignalGroupSettingsData> getSignalGroupSettingsDataByGroupId() {
		return this.signalGroupSettingsBySignalGroupId;
	}

	@Override
	public Double getStartTime() {
		return this.starttime;
	}

	@Override
	public void setCycleTime(Integer cycleTime) {
		this.cycletime = cycleTime;
	}

	@Override
	public void setEndTime(Double seconds) {
		this.endtime = seconds;
	}

	@Override
	public void setOffset(Integer seconds) {
		this.offset = seconds;
	}

	@Override
	public void setStartTime(Double seconds) {
		this.starttime = seconds;
	}

}
