/* *********************************************************************** *
 * project: org.matsim.*
 * Identifier.java
 *                                                                         *
 * *********************************************************************** *
 *                                                                         *
 * copyright       : (C) 2008 by the members listed in the COPYING,        *
 *                   LICENSE and WARRANTY file.                            *
 * email           : info at matsim dot org                                *
 *                                                                         *
 * *********************************************************************** *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *   See also COPYING, LICENSE and WARRANTY file                           *
 *                                                                         *
 * *********************************************************************** */

package org.matsim.withinday.replanning.identifiers.interfaces;

import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

import org.matsim.api.core.v01.Id;
import org.matsim.core.mobsim.qsim.agents.PlanBasedWithinDayAgent;

/**
 * Identifies agents that need a replanning of their scheduled plan.
 * 
 * @author cdobler
 */
public abstract class Identifier {
	
	private IdentifierFactory identifierFactory;
	private final Set<AgentFilter> agentFilters = new LinkedHashSet<AgentFilter>();
	
	public abstract Set<PlanBasedWithinDayAgent> getAgentsToReplan(double time);
	
	public final void addAgentFilter(AgentFilter agentFilter) {
		this.agentFilters.add(agentFilter);
	}
	
	public final boolean removeAgentFilter(AgentFilter agentFilter) {
		return this.agentFilters.remove(agentFilter);
	}

	public final Set<AgentFilter> getAgentFilters() {
		return Collections.unmodifiableSet(agentFilters);
	}
	
	public final void applyFilters(Set<Id> set, double time) {
		for (AgentFilter agentFilter : agentFilters) agentFilter.applyAgentFilter(set, time);
	}
	
	public final void setIdentifierFactory(IdentifierFactory factory) {
		this.identifierFactory = factory;
	}
	
	public final IdentifierFactory getIdentifierFactory() {
		return identifierFactory;
	}
}
